@@ -299,7 +299,7 @@ _XcursorFileHeaderDestroy (XcursorFileHeader *fileHeader)
 }
 
 static XcursorFileHeader *
-_XcursorFileHeaderCreate (XcursorUInt ntoc)
+_XcursorFileHeaderCreate (int ntoc)
 {
     XcursorFileHeader	*fileHeader;
 
